package cyberdeliaengine

class Matrix3D() {
    var _e = FloatArray(16)

    init {
        _e[0] = 1.0f
        _e[1] = 0.0f
        _e[2] = 0.0f
        _e[3] = 0.0f

        _e[4] = 0.0f
        _e[5] = 1.0f
        _e[6] = 0.0f
        _e[7] = 0.0f

        _e[8] = 0.0f
        _e[9] = 0.0f
        _e[10] = 1.0f
        _e[11] = 0.0f

        _e[12] = 0.0f
        _e[13] = 0.0f
        _e[14] = 0.0f
        _e[15] = 1.0f
    }

    fun inverse(mat: Matrix3D): Matrix3D
    {
        val a = mat._e[0]
        val b = mat._e[1]
        val c = mat._e[2]
        val d = mat._e[3]
        val e = mat._e[4]
        val f = mat._e[5]
        val g = mat._e[6]
        val h = mat._e[7]
        val i = mat._e[8]
        val j = mat._e[9]
        val k = mat._e[10]
        val l = mat._e[11]
        val m = mat._e[12]
        val n = mat._e[13]
        val o = mat._e[14]
        val p = mat._e[15]
        val q = a * f - b * e
        val r = a * g - c * e
        val s = a * h - d * e
        val t = b * g - c * f
        val u = b * h - d * f
        val v = c * h - d * g
        val w = i * n - j * m
        val x = i * o - k * m
        val y = i * p - l * m
        val z = j * o - k * n
        val A = j * p - l * n
        val B = k * p - l * o
        val ivd = 1 / (q * B - r * A + s * z + t * y - u * x + v * w)
        _e[0]  = ( f * B - g * A + h * z) * ivd
        _e[1]  = (-b * B + c * A - d * z) * ivd
        _e[2]  = ( n * v - o * u + p * t) * ivd
        _e[3]  = (-j * v + k * u - l * t) * ivd
        _e[4]  = (-e * B + g * y - h * x) * ivd
        _e[5]  = ( a * B - c * y + d * x) * ivd
        _e[6]  = (-m * v + o * s - p * r) * ivd
        _e[7]  = ( i * v - k * s + l * r) * ivd
        _e[8]  = ( e * A - f * y + h * w) * ivd
        _e[9]  = (-a * A + b * y - d * w) * ivd
        _e[10] = ( m * u - n * s + p * q) * ivd
        _e[11] = (-i * u + j * s - l * q) * ivd
        _e[12] = (-e * z + f * x - g * w) * ivd
        _e[13] = ( a * z - b * x + c * w) * ivd
        _e[14] = (-m * t + n * r - o * q) * ivd
        _e[15] = ( i * t - j * r + k * q) * ivd
        return this
    }

    fun inverse(mat: FloatArray): Matrix3D
    {
        val a = mat[0]
        val b = mat[1]
        val c = mat[2]
        val d = mat[3]
        val e = mat[4]
        val f = mat[5]
        val g = mat[6]
        val h = mat[7]
        val i = mat[8]
        val j = mat[9]
        val k = mat[10]
        val l = mat[11]
        val m = mat[12]
        val n = mat[13]
        val o = mat[14]
        val p = mat[15]
        val q = a * f - b * e
        val r = a * g - c * e
        val s = a * h - d * e
        val t = b * g - c * f
        val u = b * h - d * f
        val v = c * h - d * g
        val w = i * n - j * m
        val x = i * o - k * m
        val y = i * p - l * m
        val z = j * o - k * n
        val A = j * p - l * n
        val B = k * p - l * o
        val ivd = 1 / (q * B - r * A + s * z + t * y - u * x + v * w)
        _e[0]  = ( f * B - g * A + h * z) * ivd
        _e[1]  = (-b * B + c * A - d * z) * ivd
        _e[2]  = ( n * v - o * u + p * t) * ivd
        _e[3]  = (-j * v + k * u - l * t) * ivd
        _e[4]  = (-e * B + g * y - h * x) * ivd
        _e[5]  = ( a * B - c * y + d * x) * ivd
        _e[6]  = (-m * v + o * s - p * r) * ivd
        _e[7]  = ( i * v - k * s + l * r) * ivd
        _e[8]  = ( e * A - f * y + h * w) * ivd
        _e[9]  = (-a * A + b * y - d * w) * ivd
        _e[10] = ( m * u - n * s + p * q) * ivd
        _e[11] = (-i * u + j * s - l * q) * ivd
        _e[12] = (-e * z + f * x - g * w) * ivd
        _e[13] = ( a * z - b * x + c * w) * ivd
        _e[14] = (-m * t + n * r - o * q) * ivd
        _e[15] = ( i * t - j * r + k * q) * ivd
        return this
    }
}
